\NixieClk.xpl	28-Jun-2011	Loren Blaney	loren.blaney@gmail.com
\Nixie Clock - displays current time of day
\Compile with 32-bit XPL (xpx.bat)

\REVISIONS:
\27-Jan-2011, Original.
\28-Jun-2011, Display in five different graphic modes.

inc	c:\cxpl\codes;		\include intrinsic code declarations

int	VidMode, ScrW, ScrH,	\graphic mode, screen width & height (pixels)
	Scaler,			\needed to squeeze pizels into 320x200 screen
	WinX, WinY,		\for centering window on the screen
	PenX, PenY;		\pen location for plotting pixels
int	Tube, Digit(10);	\graphic image data arrays
char	Palette;		\array of 256 palette colors (RGB)

def	TubeW=67, GapW=TubeW/2;	\tube width and gap between hours, minutes, sec.



proc	DrawFilledRectangle(X0, Y0, W, H, C);	\Draw a filled rectangle
int	X0, Y0, W, H,	\upper left corner coordinates, width, height (pixels)
	C;		\color
int	Y;
begin
for Y:= Y0, Y0+H-1 do
	[Move(X0, Y);  Line(X0+W-1, Y, C)];
end;	\DrawFilledRectangle



proc	DrawSprite(X0, Y0, Spr); \Draw a sprite
int	X0, Y0;			\coordinates of upper-left corner (pixels)
char	Spr;			\address of sprite data
int	X, Y, I, C, W, H;
begin
W:= Spr(0);			\get width and height (in pixels)
H:= Spr(1);
I:= 4;				\point to image data
for Y:= Y0, Y0+H-1 do
    for X:= X0, X0+W-1 do
	begin
	C:= Spr(I);		\get pixel's color
	if C then		\background (0) is transparent
		Point(X>>Scaler, Y>>Scaler, C);
	I:= I+1;
	end;
end;	\DrawSprite



proc	NumOut(N);		\Draw a 2-digit number, including leading zero
int	N;
begin
DrawSprite(PenX, PenY, Digit(N/10));
PenX:= PenX+TubeW;
DrawSprite(PenX, PenY, Digit(rem(N/10)));
PenX:= PenX+TubeW;
end;	\NumOut



proc	ShowTime;		\Display current time (e.g: 15 03 45)
int	Reg;
begin
Reg:= GetReg;			\address of array that has copy of CPU registers
Reg(0):= $2C00;			\call DOS function 2C (hex)
SoftInt($21);			\DOS calls are interrupt 21 (hex)
NumOut(Reg(2) >> 8);		\the high byte of register CX contains the hours
PenX:= PenX + GapW;
NumOut(Reg(2) & $00FF);		\the low byte of CX contains the minutes
PenX:= PenX + GapW;
NumOut(Reg(3) >> 8);		\the high byte of DX contains the seconds
end;	\ShowTime



int	I, W, H;		\video mode, index, width, height

begin	\Main
include NixieClk.inc;		\nixie tube image data

Text(0, "^
                      ::::::::::::::::::::::::::::
                      ::                        ::
                      ::  Select display mode   ::
                      ::                        ::
                      ::      0:  320x200       ::
                      ::      1:  640x480       ::
                      ::      2:  800x600       ::
                      ::      3: 1024x768       ::
                      ::      4: 1280x1024      ::
                      ::                        ::
                      ::::::::::::::::::::::::::::

");
loop	begin
	OpenI(0);
	case ChIn(0) of
	  ^0:	[VidMode:=  $13;  ScrW:= 320;  ScrH:= 200;  Scaler:= 1;  quit];
	  ^1:	[VidMode:= $101;  ScrW:= 640;  ScrH:= 480;  Scaler:= 0;  quit];
	  ^2:	[VidMode:= $103;  ScrW:= 800;  ScrH:= 600;  Scaler:= 0;  quit];
	  ^3:	[VidMode:= $105;  ScrW:=1024;  ScrH:= 768;  Scaler:= 0;  quit];
	  ^4:	[VidMode:= $107;  ScrW:=1280;  ScrH:=1024;  Scaler:= 0;  quit]
	other	ChOut(0, 7);	\Oh, please!
	end;

SetVid(VidMode);		\set up graphic display
port($3C8):= 0;			\set up color registers
for I:= 0, 3*256-1 do port($3C9):= Palette(I);

W:= (TubeW*6 + 4*GapW) >> Scaler;
H:= 200 >> Scaler;
WinX:= (ScrW-W)/2;		\center window on screen
WinY:= (ScrH-H)/2;
DrawFilledRectangle(WinX-12, WinY-12, W+24, H+24, $01\white\);	\frame
DrawFilledRectangle(WinX, WinY, W, H, $23\gray\);

PenY:= WinY + 30<<Scaler;	\top edge of window + offset to tube
PenX:= WinX + GapW;		\left edge of window + offset to tube
for I:= 0, 6-1 do		\draw the tubes
	begin
	DrawSprite(PenX, PenY, Tube);
	PenX:= PenX + TubeW;	\move to next tube position
	if I=1 ! I=3 then PenX:= PenX + GapW;
	end;

PenY:= PenY + 55;		\offset to numbers inside tube
repeat	PenX:= WinX + GapW + 10;\left edge of window + offset to number
	ShowTime;		\call routine to display the current time
until KeyHit;			\keep repeating until a key is struck

SetVid($03);			\restore normal text display (for DOS)
end;	\Main
